<?php
/*
**************************************************************************

Plugin Name: Rearrange Woocommerce Products
Description: a plugin to Rearrange Woocommerce Products listed on the Shop page
Version: 1.0
Author: Aslam Doctor
Author URI: http://aslamdoctor.com
Text Domain:  rwpp
License: GPL2

**************************************************************************

Rearrange Woocommerce Products is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.
 
Rearrange Woocommerce Products is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with Rearrange Woocommerce Products. If not, see <http://www.gnu.org/licenses/>.

**************************************************************************
*/

//====== Add Admin Menu ======
add_action('admin_menu', 'rwpp_register_my_custom_submenu_page', 99);

function rwpp_register_my_custom_submenu_page() {
	add_submenu_page( 
		'edit.php?post_type=product', 
		__('Rearrange Products', 'rwpp'), 
		__('Rearrange Products', 'rwpp'), 
		'manage_options', 
		'rwpp-page', 
		'rwpp_callback' 
	);   
}


//====== Load Admin Page View ======
function rwpp_callback() {
	include "view.php";
}


//====== Add Scripts and Styles ======
add_action( 'admin_enqueue_scripts', 'rwpp_enqueue' );
function rwpp_enqueue($hook) {
    if ( 'product_page_rwpp-page' != $hook ) {
        return;
    }

    // Stylesheets
	wp_register_style('rwpp_app_css', (plugin_dir_url( __FILE__ )."/css/app.css"), false);
	wp_enqueue_style('rwpp_app_css');

	// Javascripts
	wp_register_script('rwpp_app_js', (plugin_dir_url( __FILE__ )."/js/app.js"), array(), false, true);
	wp_enqueue_script('jquery');
	wp_enqueue_script('jquery-ui-sortable');
	wp_enqueue_script('rwpp_app_js');
}



//====== Submit Form Action ======
add_action('wp_ajax_save_rwpp', 'rwpp_handle_form_action');
add_action('wp_ajax_nopriv_save_rwpp', 'rwpp_handle_form_action'); 
function rwpp_handle_form_action(){
	if ( ! empty( $_POST ) ) {
		// Sanitize post values
		$product_ids = isset( $_POST['product_id'] ) ? 
					array_map( 'sanitize_text_field', wp_unslash( $_POST['product_id'] ) ):
					array();
					
		$current_menu_orders = isset( $_POST['current_menu_order'] ) ? 
					array_map( 'sanitize_text_field', wp_unslash( $_POST['current_menu_order'] ) ):
					array();

		$new_menu_orders = isset( $_POST['new_menu_order'] ) ? 
					array_map( 'sanitize_text_field', wp_unslash( $_POST['new_menu_order'] ) ):
					array();
		
		$i=0;


		if ( is_array( $product_ids ) && is_array( $current_menu_orders ) && is_array( $new_menu_orders )) {
			foreach($product_ids as $product_id){
				$product_id = intval($product_id);
				$current_menu_order = intval($current_menu_orders[$i]);
				$new_menu_order = intval($new_menu_orders[$i]);
				
				// Update record
				$my_post = array(
					'ID'           => $product_id,
					'menu_order'   => $new_menu_order
				);
				wp_update_post( $my_post );

				// Reset post array
				$my_post="";
				
				$i++;
			}
			
			_e('<div class="notice notice-success is-dismissible">
			<p><strong>All products are rearranged now.</strong></p>
			</div>', 'rwpp'); 
		}
	}
	
	exit;
}


//====== Check if woocommerce is installed and activated before activating plugin ======
add_action( 'admin_init', 'rwpp_check_required_plugin' );
function rwpp_check_required_plugin() {
    if ( is_admin() && current_user_can( 'activate_plugins' ) &&  !is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
        add_action( 'admin_notices', 'rwpp_plugin_notice' );

        deactivate_plugins( plugin_basename( __FILE__ ) ); 

        if ( isset( $_GET['activate'] ) ) {
            unset( $_GET['activate'] );
        }
    }
}

function rwpp_plugin_notice(){
	_e('<div class="error"><p>Please activate Woocommerce plugin before using <strong>Rearrange Woocommerce Products</strong> plugin.</p></div>', 'rwpp');
}